(function(wp) {
    var el = wp.element.createElement;
    var InspectorControls = wp.editor.InspectorControls;
    var RichText = wp.editor.RichText;
    var Fragment = wp.element.Fragment;
    var MediaUpload = wp.editor.MediaUpload;
    var TextControl = wp.components.TextControl;
    var RangeControl = wp.components.RangeControl;
    var CheckboxControl = wp.components.ToggleControl;
    var SelectControl = wp.components.SelectControl;
    var ButtonGroup = wp.components.ButtonGroup;
    var Button = wp.components.Button;
    var ColorPalette = wp.components.ColorPalette;
    var ServerSideRender = wp.components.ServerSideRender;
    var __ = wp.i18n.__;
	
	var categorySelections = [];
	var allCategories = wp.apiFetch({path: "/wp/v2/categories"}).then(posts => {
		categorySelections.push({label: __('Select a category', 'cosmic-blocks'), value: 0});
		jQuery.each( posts, function( key, val ) {
			categorySelections.push({label: val.name, value: val.id});
		});
		return categorySelections;
	});

    wp.blocks.registerBlockType('cwp/posts-list', {
        title: __('Posts List', 'cosmic-blocks'),
		keywords: [
			'posts list',
			__('articles list', 'cosmic-blocks'),
			__('news list', 'cosmic-blocks'),
		],
        icon: {
			background: "#16A3FF",
			foreground: "#FFFFFF",
			src: "editor-justify"
		},
        category: 'cwp-blocks',
        attributes: {
			preview: {
				type: 'boolean',
				default: false
			},
			blockID: {
				type: 'string',
				default: ''
			},
            columns: {
                type: "number",
                default: 1
            },
            container: {
                type: "string",
                default: 'cwp-full-width-wide-container'
            },
			startClass: {
                type: "string",
                default: 'cwp-all-left'
            },
            backgroundType: {
                type: "string",
                default: 'color'
            },
            backgroundTextColor: {
                type: "string",
                default: ''
            },
            solidColor: {
                type: "string",
                default: ''
            },
            gradient: {
                type: "string"
            },
            backgroundMediaID: {
                type: "number"
            },
            backgroundMediaURL: {
                type: "string"
            },
            animationStyle: {
                type: "string"
            },
            animationDelay: {
                type: "number",
                default: 10
            },
            threshold: {
                type: "number",
                default: 40
            },
            paddingBeforeIntro: {
                type: 'number',
                default: 50
            },
            paddingAfterIntro: {
                type: 'number',
                default: 50
            },
            paddingAfterItems: {
                type: 'number',
                default: 50
            },
			introsection: {
                type: 'string',
                introdefault: 'empty'
            },
            introtitle: {
                type: "string",
                default: ''
            },
            introsubtitle: {
                type: "string",
                default: ''
            },
            introduction: {
                type: "string",
                default: ''
            },
			boxradius: {
                type: "number",
                default: 0
            },
			boxradiusfix: {
				type: 'boolean',
				default: false
			},
            boxpadding: {
                type: "number",
                default: 0
            },
			boxpaddingfix: {
				type: 'boolean',
				default: false
			},
            postsPerPage: {
                type: "string",
                default: '25'
            },
            position: {
                type: "string",
                default: 'inside'
            },
            order: {
                type: "string",
                default: 'ASC'
            },
            orderby: {
                type: "string",
                default: 'date'
            },
            showTitle: {
                type: "boolean",
                default: true
            },
			showExcerpt: {
                type: "boolean",
                default: true
            },
			excerptCharlength: {
                type: "string",
                default: '120'
            },
			showContent: {
                type: "boolean",
                default: false
            },
            showDate: {
                type: "boolean",
                default: true
            },
            showAuthor: {
                type: "boolean",
                default: true
            },
            showImage: {
                type: "boolean",
                default: true
            },
			imageSize: {
				type: "string",
				default: ""
			},
            showButton: {
                type: "string",
                default: 'yes'
            },
            showButtonText: {
                type: "string",
                default: __('Read more', 'cosmic-blocks')
            },
            showButtonStyle: {
                type: "string",
                default: "cwp-button cwp-button-secondary"
            },
            showButtonTarget: {
                type: "string",
                default: "_self"
            },
			selectedCategory: {
				type: "string",
                default: ""
			},
			excludedCategory: {
				type: "string",
                default: ""
			},
			showPagination: {
                type: "boolean",
                default: true
            },
			paginationStyle: {
                type: "string",
                default: "cwp-pagination cwp-pagination-secondary"
            },
            textcolor: {
                type: "string",
                default: ""
            },
            color: {
                type: "string",
                default: ""
            }
        },
        getEditWrapperProps({
            container
        }) {
            if ('cwp-full-width-wide-container' === container || 'cwp-full-width-small-container' === container || 'cwp-same-width-full-container' === container) {
                return {
                    'data-align': 'full'
                };
            } else if ('cwp-same-width-wide-container' === container) {
                return {
                    'data-align': 'wide'
                };
            }
        },
        edit: function(props) {
            var attributes = props.attributes;
			
			// override default params for new entries only
			if(attributes.introsection === 'empty' || attributes.introsection === undefined)
			{
				attributes.paddingBeforeIntro = -1;
				attributes.paddingAfterIntro = -1;
				attributes.paddingAfterItems = -1;
				attributes.boxradius = -1;
				attributes.boxradiusfix = true;
				attributes.boxpadding = -1;
				attributes.boxpaddingfix = true;
				attributes.container = 'cwp-full-width-small-container';
			}
			
			// disable introsection for new entries only
			if(attributes.introsection === 'empty' || attributes.introsection === undefined)
			{				
				if(
					(attributes.introtitle !== '' && attributes.introtitle !== undefined) ||
					(attributes.introsubtitle !== '' && attributes.introsubtitle !== undefined) ||
					(attributes.introduction !== '' && attributes.introduction !== undefined))
				{
					attributes.introsection = 'enabled';
				} else {
					attributes.introsection = 'disabled';
				}
			}
			
            var solidColorPreset = el(ColorPalette, {
				value: attributes.solidColor,
				onChange: function(e) {
					props.setAttributes({
						solidColor: e
					});
				},
				colors: [{
					color: "#ffffff",
					name: "white"
				}, {
					color: "#00d1b2",
					name: "teal"
				}, {
					color: "#3373dc",
					name: "royal blue"
				}, {
					color: "#209cef",
					name: "sky blue"
				}, {
					color: "#22d25f",
					name: "green"
				}]
			});
			var backgroundTextColorPreset = el(ColorPalette, {
				value: attributes.backgroundTextColor,
				onChange: function(e) {
					props.setAttributes({
						backgroundTextColor: e
					});
				},
				colors: [{
					color: "#ffffff",
					name: "white"
				}, {
					color: "#333333",
					name: "dark"
				}, {
					color: "#000000",
					name: "black"
				}, {
					color: "#209cef",
					name: "sky blue"
				}, {
					color: "#22d25f",
					name: "green"
				}]
			});
			var gradientPreset = el(SelectControl, {
				label: __('Gradients', 'cosmic-blocks'),
				options: gradient_options.map(function(e) {
					return {
						value: e.colors,
						label: e.name
					};
				}),
				value: attributes.gradient,
				onChange: function(e) {
					props.setAttributes({
						gradient: e
					});
				}
			});
			var back_style = function() {
				var s = {};
				if (attributes.backgroundType === 'color' && attributes.solidColor !== '' && attributes.solidColor !== undefined) {
					s = {
						background: attributes.solidColor
					};
				} else if (attributes.backgroundType === 'gradient' && attributes.gradient !== '' && attributes.gradient !== undefined) {
					s = {
						background: "linear-gradient(to bottom, " + attributes.gradient + ")"
					};
				} else if (attributes.backgroundType === 'image' && attributes.backgroundMediaURL !== '' && attributes.backgroundMediaURL !== undefined) {
					s = {
						background: "url( " + attributes.backgroundMediaURL + " )",
						backgroundSize: "cover",
						backgroundPosition: "center"
					};
				}
				return s;
			};
			var back_text_style = function() {
				var s = {};
				if (attributes.backgroundTextColor !== '' && attributes.backgroundTextColor !== undefined) {
					s = {
						color: attributes.backgroundTextColor
					};
				}
				return s;
			};
			block_back_style = function() {
				var s = {};
				if(attributes.color !== '' && attributes.color !== undefined) {
					s = {
						background: attributes.color
					};
				}
				return s;
			};
			var block_text_style = function() {
				var s = {};
				if(attributes.textcolor !== '' && attributes.textcolor !== undefined) {
					s = {
						color: attributes.textcolor
					};
				}
				return s;
			};
			var box_padding = function() {
				var s = {};
				if (attributes.paddingBeforeIntro >= 0) {
					s.paddingTop = attributes.paddingBeforeIntro;
				}
				if (attributes.paddingAfterItems >= 0) {
					s.paddingBottom = attributes.paddingAfterItems;
				}
				return s;
			};

            return [
                el(
                    Fragment,
                    null,
                    el(InspectorControls, {
                            key: 'inspector'
                        }, // Display the block options in the inspector panel.
                        el(wp.components.PanelBody, {
								title: __('Intro', 'cosmic-blocks'),
								className: 'cwp-inspector-intro',
								initialOpen: false
							},
							el(SelectControl, {
								label: __('Enable/disable', 'cosmic-blocks'),
								value: attributes.introsection,
								options: [{
										label: __('Disabled', 'cosmic-blocks'),
										value: 'disabled'
									},
									{
										label: __('Enabled', 'cosmic-blocks'),
										value: 'enabled'
									}
								],
								onChange: function onChange(value) {
									props.setAttributes({
										introsection: value
									});
								}
							}),
							attributes.introsection === 'enabled' ? el('p', {}, __('Intro text color', 'cosmic-blocks')) : '',
							attributes.introsection === 'enabled' ? backgroundTextColorPreset : '',
							attributes.introsection === 'enabled' ? el(RangeControl, {
								label: __('Padding after intro text', 'cosmic-blocks'),
								value: attributes.paddingAfterIntro,
								allowReset: false,
								initialPosition: 50,
								onChange: function(e) {
									return props.setAttributes({
										paddingAfterIntro: e
									});
								},
								min: -1,
								max: 200
							}) : '',
						),
                        el(wp.components.PanelBody, {
                                title: __('Layout', 'cosmic-blocks'),
                                className: 'cwp-inspector-layout',
                                initialOpen: false
							},
                            el(SelectControl, {
                                label: __('Container width', 'cosmic-blocks'),
                                value: attributes.container,
                                options: container_options.map(function(e) {
                                    return {
                                        value: e.value,
                                        label: e.label
                                    };
                                }),
                                onChange: function(e) {
                                    props.setAttributes({
                                        container: e
                                    });
                                }
                            }),
                            el(RangeControl, {
                                label: __('Padding top', 'cosmic-blocks'),
                                value: attributes.paddingBeforeIntro,
                                allowReset: false,
                                initialPosition: 50,
                                onChange: function(e) {
                                    return props.setAttributes({
                                        paddingBeforeIntro: e
                                    });
                                },
                                min: -1,
                                max: 200
                            }),
                            el(RangeControl, {
                                label: __('Padding bottom', 'cosmic-blocks'),
                                value: attributes.paddingAfterItems,
                                allowReset: false,
                                initialPosition: 50,
                                onChange: function(e) {
                                    return props.setAttributes({
                                        paddingAfterItems: e
                                    });
                                },
                                min: -1,
                                max: 200
                            }),
                            el('p', {
                                className: 'cwp-custom-label'
                            }, __('Background', 'cosmic-blocks')),
                            el(ButtonGroup, {},
                                backgroundType.map(function(e) {
                                    return el(Button, {
                                        key: e.label,
                                        isLarge: !0,
                                        isPrimary: attributes.backgroundType === e.value,
                                        "aria-pressed": attributes.backgroundType === e.value,
                                        onClick: function() {
                                            return props.setAttributes({
                                                backgroundType: e.value
                                            });
                                        }
                                    }, e.label);
                                })
                            ),
                            "color" === attributes.backgroundType && solidColorPreset,
                            "gradient" === attributes.backgroundType && gradientPreset,
                            "image" === attributes.backgroundType && el(
                                MediaUpload, {
                                    key: "mediaupload",
                                    onSelect: function(media) {
                                        return props.setAttributes({
                                            backgroundMediaURL: media.url,
                                            backgroundMediaID: media.id
                                        });
                                    },
                                    type: "image",
                                    value: attributes.backgroundMediaID,
                                    render: function(e) {
                                        var r = e.open;
                                        return el(Fragment, null, el(Button, {
                                            className: "button-link cwp-custom-button",
                                            onClick: r
                                        }, attributes.backgroundMediaID ? el("img", {
                                            src: attributes.backgroundMediaURL
                                        }) : __('Set background image', 'cosmic-blocks')));
                                    }
                                }
                            ),
                            "image" === attributes.backgroundType && attributes.backgroundMediaID && el(Fragment, null, el("p", {
                                className: "cwp-custom-edit-image"
                            }, __('Click the image to edit or update', 'cosmic-blocks')), el(Button, {
                                className: "button-link",
                                style: {
                                    marginBottom: "20px"
                                },
                                onClick: function() {
                                    return props.setAttributes({
                                        backgroundMediaURL: void 0,
                                        backgroundMediaID: void 0
                                    });
                                }
                            }, __('Remove background image', 'cosmic-blocks'))),
                            el(TextControl, {
                                label: __('Block ID', 'cosmic-blocks'),
                                value: attributes.blockID,
                                onChange: function(e) {
                                    return props.setAttributes({
                                        blockID: e
                                    });
                                }
                            }),
						),
						el(wp.components.PanelBody, {
                                title: __('Items', 'cosmic-blocks'),
                                className: 'cwp-inspector-layout',
                                initialOpen: true
                            },
							el('div', {
									className: ''
								},
								el(SelectControl,
									{
										label: __('By category', 'cosmic-blocks'),
										options: categorySelections,
										value: attributes.selectedCategory,
										onChange: function(e) {
											return props.setAttributes({
												selectedCategory: e
											});
										}
									}
								)
							),
							el('div', {
									className: ''
								},
								el(SelectControl,
									{
										label: __('Exclude category', 'cosmic-blocks'),
										options: categorySelections,
										value: attributes.excludedCategory,
										onChange: function(e) {
											return props.setAttributes({
												excludedCategory: e
											});
										}
									}
								)
							),
							el('div', {
									className: ''
								},
								el(TextControl, {
									type: 'number',
									label: __('Posts per page', 'cosmic-blocks'),
									value: attributes.postsPerPage,
									onChange: function(e) {
										return props.setAttributes({
											postsPerPage: e
										});
									}
								}),
							),
							el('div', {
									className: ''
								},
								el(SelectControl, {
									label: __('Order by', 'cosmic-blocks'),
									value: attributes.orderby,
									options: [{
											label: __('None', 'cosmic-blocks'),
											value: 'none'
										},
										{
											label: __('ID', 'cosmic-blocks'),
											value: 'ID'
										},
										{
											label: __('Author', 'cosmic-blocks'),
											value: 'author'
										},
										{
											label: __('Title', 'cosmic-blocks'),
											value: 'title'
										},
										{
											label: __('Date', 'cosmic-blocks'),
											value: 'date'
										},
										{
											label: __('Modified', 'cosmic-blocks'),
											value: 'modified'
										},
										{
											label: __('Parent', 'cosmic-blocks'),
											value: 'parent'
										},
										{
											label: __('Rand', 'cosmic-blocks'),
											value: 'rand'
										},
										{
											label: __('Comment count', 'cosmic-blocks'),
											value: 'comment_count'
										},
										{
											label: __('Menu order', 'cosmic-blocks'),
											value: 'menu_order'
										}
									],
									onChange: function(e) {
										props.setAttributes({
											orderby: e
										});
									}
								})
							),
							el('div', {
									className: ''
								},
								el(SelectControl, {
									label: __('Order', 'cosmic-blocks'),
									value: attributes.order,
									options: [{
											label: __('ASC', 'cosmic-blocks'),
											value: 'ASC'
										},
										{
											label: __('DESC', 'cosmic-blocks'),
											value: 'DESC'
										}
									],
									onChange: function(e) {
										props.setAttributes({
											order: e
										});
									}
								})
							),
							el('div', {
									className: ''
								},
								el(SelectControl, {
									label: __('Title and meta position', 'cosmic-blocks'),
									value: attributes.position,
									options: [{
											label: __('Before', 'cosmic-blocks'),
											value: 'before'
										},
										{
											label: __('Inside', 'cosmic-blocks'),
											value: 'inside'
										}
									],
									onChange: function(e) {
										props.setAttributes({
											position: e
										});
									}
								})
							),
							el('div', {
									className: ''
								},
								el(SelectControl, {
									label: __('Start position', 'cosmic-blocks'),
									value: attributes.startClass,
									options: [{
											label: __('Start left', 'cosmic-blocks'),
											value: 'cwp-start-left'
										},
										{
											label: __('Start right', 'cosmic-blocks'),
											value: 'cwp-start-right'
										},
										{
											label: __('All left', 'cosmic-blocks'),
											value: 'cwp-all-left'
										},
										{
											label: __('All right', 'cosmic-blocks'),
											value: 'cwp-all-right'
										}
									],
									onChange: function onChange(value) {
										props.setAttributes({
											startClass: value
										});
									}
								})
							),
							el('p', {
								style: block_text_style()
							}, __('Show or hide items', 'cosmic-blocks')),
							el('div', {
									className: 'cwp-control'
								},
								el(CheckboxControl, {
									label: __('Title', 'cosmic-blocks'),
									checked: attributes.showTitle,
									onChange: function(e) {
										return props.setAttributes({
											showTitle: e
										});
									}
								}),
							),
							el('div', {
									className: 'cwp-control'
								},
								el(CheckboxControl, {
									label: __('Excerpt', 'cosmic-blocks'),
									checked: attributes.showExcerpt,
									onChange: function(e) {
										return props.setAttributes({
											showExcerpt: e
										});
									}
								}),
							),
							attributes.showExcerpt === true ? el('div', {
									className: ''
								},
								el(TextControl, {
									type: 'number',
									label: __('Excerpt charlength', 'cosmic-blocks'),
									value: attributes.excerptCharlength,
									onChange: function(e) {
										return props.setAttributes({
											excerptCharlength: e
										});
									}
								}),
							) : '',
							el('div', {
									className: 'cwp-control'
								},
								el(CheckboxControl, {
									label: __('Content', 'cosmic-blocks'),
									checked: attributes.showContent,
									onChange: function(e) {
										return props.setAttributes({
											showContent: e
										});
									}
								}),
							),
							el('div', {
									className: 'cwp-control'
								},
								el(CheckboxControl, {
									label: __('Date', 'cosmic-blocks'),
									checked: attributes.showDate,
									onChange: function(e) {
										return props.setAttributes({
											showDate: e
										});
									}
								}),
							),
							el('div', {
									className: 'cwp-control'
								},
								el(CheckboxControl, {
									label: __('Author', 'cosmic-blocks'),
									checked: attributes.showAuthor,
									onChange: function(e) {
										return props.setAttributes({
											showAuthor: e
										});
									}
								}),
							),
							el('div', {
									className: 'cwp-control'
								},
								el(CheckboxControl, {
									label: __('Image', 'cosmic-blocks'),
									checked: attributes.showImage,
									onChange: function(e) {
										return props.setAttributes({
											showImage: e
										});
									}
								}),
							),
							attributes.showImage === true ? el('div', {
									className: ''
								},
								el(SelectControl, {
									label: __('Image size', 'cosmic-blocks'),
									value: attributes.imageSize,
									options: thumbnail_options.map(function(e) {
										return {
											value: e.value,
											label: e.label
										};
									}),
									onChange: function(e) {
										props.setAttributes({
											imageSize: e
										});
									}
								}),
							) : '',
							el('div', {
									className: 'cwp-control'
								},
								el(CheckboxControl, {
									label: __('Pagination', 'cosmic-blocks'),
									checked: attributes.showPagination,
									onChange: function(e) {
										return props.setAttributes({
											showPagination: e
										});
									}
								}),
							),
							attributes.showPagination === true ? el('div', {
									className: ''
								},
								el(SelectControl, {
									label: __('Pagination style', 'cosmic-blocks'),
									value: attributes.paginationStyle,
									options: pagination_options.map(function(e) {
										return {
											value: e.value,
											label: e.label
										};
									}),
									onChange: function onChange(value) {
										return props.setAttributes({
											paginationStyle: value
										});
									}
								})
							): '',
							el('hr'),
							el('div', {
									className: ''
								},
								el(SelectControl, {
									label: __('Show Button', 'cosmic-blocks'),
									value: attributes.showButton,
									options: [{
											label: __('Yes', 'cosmic-blocks'),
											value: 'yes'
										},
										{
											label: __('No', 'cosmic-blocks'),
											value: 'no'
										}
									],
									onChange: function(e) {
										props.setAttributes({
											showButton: e
										});
									}
								})
							),
							attributes.showButton === 'yes' ? el('div', {
									className: ''
								},
								el(SelectControl, {
									value: attributes.showButtonStyle,
									options: button_options.map(function(e) {
										return {
											value: e.value,
											label: e.label
										};
									}),
									onChange: function onChange(value) {
										return props.setAttributes({
											showButtonStyle: value
										});
									}
								})
							) : '',
							attributes.showButton === 'yes' ? el('div', {
									className: ''
								},
								el(SelectControl, {
									value: attributes.showButtonTarget,
									options: [{
											label: __('Open link in same window', 'cosmic-blocks'),
											value: '_self'
										},
										{
											label: __('Open link in new window', 'cosmic-blocks'),
											value: '_blank'
										}
									],
									onChange: function onChange(value) {
										return props.setAttributes({
											showButtonTarget: value
										});
									}
								})
							) : '',
							attributes.showButton === 'yes' ? el(RichText, {
								tagName: 'div',
								className: attributes.showButtonStyle,
								placeholder: '',
								keepPlaceholderOnFocus: true,
								value: attributes.showButtonText,
								multiline: false,
								isSelected: attributes.isSelected,
								formattingControls: [],
								onChange: function onChange(value) {
									return props.setAttributes({
										showButtonText: value
									});
								}
							}) : '',
							el('hr'),
							el(RangeControl, {
								label: __('Items border radius', 'cosmic-blocks'),
								value: attributes.boxradius,
								onChange: function(e) {
									return props.setAttributes({
										boxradius: e,
										boxradiusfix: true
									});
								},
								min: -1,
								max: 50
							}),
							el(RangeControl, {
								label: __('Items padding', 'cosmic-blocks'),
								value: attributes.boxpadding,
								onChange: function(e) {
									return props.setAttributes({
										boxpadding: e,
										boxpaddingfix: true
									});
								},
								min: -1,
								max: 50
							})
						),
						el(wp.components.PanelBody, {
                                title: __('Animation', 'cosmic-blocks'),
                                className: 'cwp-inspector-animation',
                                initialOpen: false
                            },
                            el(SelectControl, {
								label: __('Animation', 'cosmic-blocks'),
                                value: attributes.animationStyle,
                                options: animation_options.map(function(e) {
                                    return {
                                        value: e.value,
                                        label: e.label
                                    };
                                }),
                                onChange: function(e) {
                                    return props.setAttributes({
                                        animationStyle: e
                                    });
                                }
                            }),
                            el(TextControl, {
								label: __('Animation Delay', 'cosmic-blocks'),
                                value: attributes.animationDelay,
                                onChange: function(e) {
                                    return props.setAttributes({
                                        animationDelay: e
                                    });
                                },
                                help: __('Enter in milliseconds', 'cosmic-blocks')
                            }),
                            el(RangeControl, {
                                beforeIcon: 'arrow-left-alt2',
                                afterIcon: 'arrow-right-alt2',
                                label: __('Threshold', 'cosmic-blocks'),
                                value: attributes.threshold,
								allowReset: false,
								initialPosition: 40,
                                onChange: function(e) {
                                    return props.setAttributes({
                                        threshold: e
                                    });
                                },
                                min: 0,
                                max: 100
                            })
                        )
                    ),
                    el('div', {
							id: (attributes.blockID !== '' && attributes.blockID !== undefined ? attributes.blockID : ''),
                            className: props.className + ' ' + attributes.container + ' cwp-common-main-wrap cwp-backend-view-content' + (Object.keys(back_style()).length !== 0 ? ' cwp-common-main-wrap-has-background' : ''),
                            style: back_style()
                        },
                        el('div', {
                                className: 'cwp-common-main-wrap-inner' + (attributes.animationStyle !== '' && attributes.animationStyle !== undefined ? ' animated ' + attributes.animationStyle : ''),
                                style: box_padding()
                            },
                            attributes.introsection === 'enabled' ? el('div', {
                                    className: 'cwp-main-intro',
                                    style: attributes.paddingAfterIntro >= 0 ? {
										paddingBottom: attributes.paddingAfterIntro
									} : {}
                                },
                                el('div', {
                                        className: 'cwp-intro-title'
                                    },
                                    el('div', {
                                            className: 'cwp-intro-input'
                                        },
                                        el(RichText, {
                                            tagName: 'h3',
                                            placeholder: __('Title', 'cosmic-blocks'),
                                            keepPlaceholderOnFocus: true,
                                            value: attributes.introtitle,
                                            style: back_text_style(),
                                            isSelected: attributes.isSelected,
											formattingControls: [ 'bold', 'italic', 'strikethrough', 'link' ],
                                            onChange: function onChange(e) {
                                                return props.setAttributes({
                                                    introtitle: e
                                                });
                                            }
                                        }),
                                    ),
                                ),
                                el('div', {
                                        className: 'cwp-intro-subtitle'
                                    },
                                    el('div', {
                                            className: 'cwp-intro-input'
                                        },
                                        el(RichText, {
                                            tagName: 'p',
                                            placeholder: __('Subtitle', 'cosmic-blocks'),
                                            keepPlaceholderOnFocus: true,
                                            value: attributes.introsubtitle,
                                            style: back_text_style(),
                                            isSelected: attributes.isSelected,
											formattingControls: [ 'bold', 'italic', 'strikethrough', 'link' ],
                                            onChange: function onChange(e) {
                                                return props.setAttributes({
                                                    introsubtitle: e
                                                });
                                            }
                                        }),
                                    ),
                                ),
                                el('div', {
                                        className: 'cwp-intro-introduction'
                                    },
                                    el('div', {
                                            className: 'cwp-intro-input'
                                        },
                                        el(RichText, {
                                            tagName: 'p',
                                            placeholder: __('Introduction', 'cosmic-blocks'),
                                            keepPlaceholderOnFocus: true,
                                            value: attributes.introduction,
                                            style: back_text_style(),
                                            isSelected: attributes.isSelected,
											formattingControls: [ 'bold', 'italic', 'strikethrough', 'link' ],
                                            onChange: function onChange(e) {
                                                return props.setAttributes({
                                                    introduction: e
                                                });
                                            }
                                        }),
                                    ),
                                )
                            ) : '',
                            el('div', {
                                    className: 'cwp-main-wrapper cwp-' + attributes.columns + '-column'
                                },
                                el('div', {
                                        className: 'cwp-inner-wrap'
                                    },
                                    el('div', {
                                            className: 'cwp-meta-options'
                                        },
                                        el('div', {
                                                className: 'cwp-element-bgcolor-main'
                                            },
                                            el('div', {
                                                    className: 'cwp-drag-item cwp-option-bgcolor'
                                                },
                                                el('i', {
                                                    className: 'fa fa-paint-brush'
                                                }),
                                                el(ColorPalette, {
                                                    value: attributes.color,
                                                    onChange: function(e) {
                                                        return props.setAttributes({
                                                            color: e
                                                        });
                                                    }
                                                })
                                            )
                                        ),
                                        el('div', {
                                                className: 'cwp-element-textcolor-main'
                                            },
                                            el('div', {
                                                    className: 'cwp-drag-item cwp-option-textcolor'
                                                },
                                                el('i', {
                                                    className: 'fa fa-font'
                                                }),
                                                el(ColorPalette, {
                                                    value: attributes.textcolor,
                                                    onChange: function(e) {
                                                        return props.setAttributes({
                                                            textcolor: e
                                                        });
                                                    }
                                                })
                                            )
                                        )
                                    ),
                                    el('div', {
											className: 'cwp-server-side-render'
										},
										el(ServerSideRender, {
											block: "cwp/posts-list",
											attributes: {
												preview: true,
												containerID: attributes.containerID,
												columns: attributes.columns,
												container: attributes.container,
												startClass: attributes.startClass,
												backgroundType: attributes.backgroundType,
												backgroundTextColor: attributes.backgroundTextColor,
												solidColor: attributes.solidColor,
												gradient: attributes.gradient,
												backgroundMediaID: attributes.backgroundMediaID,
												backgroundMediaURL: attributes.backgroundMediaURL,
												animationStyle: attributes.animationStyle,
												animationDelay: attributes.animationDelay,
												threshold: attributes.threshold,
												paddingBeforeIntro: attributes.paddingBeforeIntro,
												paddingAfterIntro: attributes.paddingAfterIntro,
												paddingAfterItems: attributes.paddingAfterItems,
												introsection: attributes.introsection,
												introtitle: attributes.introtitle,
												introsubtitle: attributes.introsubtitle,
												introduction: attributes.introduction,
												boxradius: attributes.boxradius,
												boxradiusfix: attributes.boxradiusfix,
												boxpadding: attributes.boxpadding,
												boxpaddingfix: attributes.boxpaddingfix,
												postsPerPage: attributes.postsPerPage,
												position: attributes.position,
												order: attributes.order,
												orderby: attributes.orderby,
												showTitle: attributes.showTitle,
												showExcerpt: attributes.showExcerpt,
												showContent: attributes.showContent,
												showDate: attributes.showDate,
												showAuthor: attributes.showAuthor,
												showImage: attributes.showImage,
												imageSize: attributes.imageSize,
												showButton: attributes.showButton,
												showButtonText: attributes.showButtonText,
												showButtonStyle: attributes.showButtonStyle,
												showButtonTarget: attributes.showButtonTarget,
												selectedCategory: attributes.selectedCategory,
												excludedCategory: attributes.excludedCategory,
												showPagination: attributes.showPagination,
												paginationStyle: attributes.paginationStyle,
												textcolor: attributes.textcolor,
												color: attributes.color
											}
										})
									)
                                )
                            )
                        )
                    ),

                    /** Front view **/
					el('div', {
                            id: (attributes.blockID !== '' && attributes.blockID !== undefined ? attributes.blockID : ''),
                            className: props.className + ' ' + attributes.container + ' cwp-common-main-wrap cwp-front-view-content' + (Object.keys(back_style()).length !== 0 ? ' cwp-common-main-wrap-has-background' : ''),
                            style: back_style()
                        },
                        el('div', {
                                className: 'cwp-common-main-wrap-inner',
                                style: box_padding(),
                                "data-scroll-class": attributes.animationStyle,
                                "data-scroll-delay": attributes.animationDelay,
                                "data-scroll-threshold": attributes.threshold
                            },
                            attributes.introsection === 'enabled' ? el('div', {
                                    className: 'cwp-main-intro',
                                    style: attributes.paddingAfterIntro >= 0 ? {
										paddingBottom: attributes.paddingAfterIntro
									} : {}
                                },
                                (attributes.introtitle !== '' && attributes.introtitle !== undefined) || (attributes.introsubtitle !== '' && attributes.introsubtitle !== undefined) ? el('div', {
                                        className: 'cwp-intro-title'
                                    },
                                    el('h3', {
                                            style: back_text_style()
                                        },
                                        (attributes.introtitle !== '' && attributes.introtitle !== undefined) ? el(
                                            RichText.Content, {
                                                tagName: '',
                                                value: attributes.introtitle
                                            }
                                        ) : '',
                                        (attributes.introsubtitle !== '' && attributes.introsubtitle !== undefined) ? el(
                                            RichText.Content, {
                                                tagName: 'span',
                                                value: attributes.introsubtitle
                                            }
                                        ) : '',
                                    ),
                                ) : '',
                                (attributes.introduction !== '' && attributes.introduction !== undefined) ? el('div', {
                                        className: 'cwp-intro-introduction'
                                    },
                                    el('div', {
                                            className: 'cwp-intro-input'
                                        },
                                        el(
                                            RichText.Content, {
                                                tagName: 'p',
                                                value: attributes.introduction,
                                                style: back_text_style()
                                            }
                                        ),
                                    ),
                                ) : '',
                            ) : '',
							el(ServerSideRender, {
								block: "cwp/posts-list",
								attributes: {
									preview: true,
									containerID: attributes.containerID,
									columns: attributes.columns,
									container: attributes.container,
									startClass: attributes.startClass,
									backgroundType: attributes.backgroundType,
									backgroundTextColor: attributes.backgroundTextColor,
									solidColor: attributes.solidColor,
									gradient: attributes.gradient,
									backgroundMediaID: attributes.backgroundMediaID,
									backgroundMediaURL: attributes.backgroundMediaURL,
									animationStyle: attributes.animationStyle,
									animationDelay: attributes.animationDelay,
									threshold: attributes.threshold,
									paddingBeforeIntro: attributes.paddingBeforeIntro,
									paddingAfterIntro: attributes.paddingAfterIntro,
									paddingAfterItems: attributes.paddingAfterItems,
									introsection: attributes.introsection,
									introtitle: attributes.introtitle,
									introsubtitle: attributes.introsubtitle,
									introduction: attributes.introduction,
									boxradius: attributes.boxradius,
									boxradiusfix: attributes.boxradiusfix,
									boxpadding: attributes.boxpadding,
									boxpaddingfix: attributes.boxpaddingfix,
									postsPerPage: attributes.postsPerPage,
									position: attributes.position,
									order: attributes.order,
									orderby: attributes.orderby,
									showTitle: attributes.showTitle,
									showExcerpt: attributes.showExcerpt,
									showContent: attributes.showContent,
									showDate: attributes.showDate,
									showAuthor: attributes.showAuthor,
									showImage: attributes.showImage,
									imageSize: attributes.imageSize,
									showButton: attributes.showButton,
									showButtonText: attributes.showButtonText,
									showButtonStyle: attributes.showButtonStyle,
									showButtonTarget: attributes.showButtonTarget,
									selectedCategory: attributes.selectedCategory,
									excludedCategory: attributes.excludedCategory,
									showPagination: attributes.showPagination,
									paginationStyle: attributes.paginationStyle,
									textcolor: attributes.textcolor,
									color: attributes.color
								}
							})
                        )
                    )
                    /** Front view **/
                )
            ];
        },
        save: function() {
            // Rendering in PHP
            return null;
        }
    });
})(
    window.wp
);